import React, { Component } from 'react'
import Input from './input'

class Field extends Component {

  static defaultProps = {
    fieldType: 'input',
    dataType:  'string',
    validations: [],
    onChange: () => {},
  }

  // Utility Functions

  failedValidations() {
    const currentValue = this.refs.element.value()
    return this.props.validations.filter(validation => {
      return !validation.func(currentValue)
    }).map(validation => validation.description)
  }

  name() {
    return this.props.name
  }

  value() {
    return this.refs.element.value()
  }

  // Hanlder Functions

  handleChange(event) {
    this.props.onChange(event, this)
  }

  // Render Helper Functions

  fieldElementClass() {
    return {
      'input': Input
    }[this.props.fieldType]
  }

  fieldElementProps() {
    return {
      ref:          'element',
      name:         this.props.name,
      placeholder:  this.props.placeholder,
      onFormChange: this.props.onFormChange,
      onChange:     this.handleChange.bind(this),
      defaultValue: this.props.defaultValue,
      password:     this.props.password,
    }
  }

  // Render Functions

  render() {
    const elementClass = this.fieldElementClass()
    let elementProps = this.fieldElementProps()
    return React.createElement(elementClass, elementProps)
  }

}

Field.propTypes = {
  name:         React.PropTypes.string.isRequired,
  placeholder:  React.PropTypes.string,
  fieldType:    React.PropTypes.string,
  dataType:     React.PropTypes.string,
  onFormChange: React.PropTypes.func,
};


export default Field
