import React, { Component } from 'react';
import Field from './field'
import Submit from './submit'

class FormFactor extends Component {

  static defaultProps = {
    onChange: () => {},
    onSubmit: () => {},
  }

  // Utility Functions

  failedValidations() {
    return this.fields().map(field => ({
      field: field.name(),
      validations: field.failedValidations()
    })).filter(field => field.validations.length)
  }

  fields() {
    return this.fieldNames().map(fieldName => this.refs[fieldName])
  }

  fieldNames() {
    return this.props.fields.map(field => field.name)
  }

  fieldValues() {
    let values = {}
    for (const fieldName of this.fieldNames()) {
      const field = this.refs[fieldName]
      values[fieldName] = field.value()
    }
    return values
  }

  // Handler Functions

  handleChange(event) {
    this.props.onChange(event)
  }

  handleSubmit(event) {
    this.props.onSubmit(event)
  }

  // Render Functions

  renderFields() {
    return this.props.fields.map((fieldProps, index) => {
      fieldProps.key          = index
      fieldProps.ref          = fieldProps.name
      fieldProps.onFormChange = this.handleChange.bind(this)
      return React.createElement(Field, fieldProps)
    })
  }

  renderSubmit() {
    return (
      <Submit
        submitButton={this.props.submitButton}
      />
    )
  }

  render() {
    return (
      <div>
        <form
          ref='form'
          onSubmit={this.handleSubmit.bind(this)}
          >
          {this.renderFields()}
          {this.renderSubmit()}
        </form>
      </div>
    )
  }
}

FormFactor.propTypes = {
  fields: React.PropTypes.array.isRequired,
  submit: React.PropTypes.object,
  onSubmit: React.PropTypes.func,
  onChange: React.PropTypes.func,
};

export default FormFactor
